;; Generated package description from drepl.el  -*- no-byte-compile: t -*-
(define-package "drepl" "0.3.0.20240929.81709" "REPL protocol for the dumb terminal" '((emacs "29.1") (comint-mime "0.3")) :commit "d84f23f20c4a4674da957a97a8a628041f286b97" :authors '(("Augusto Stoffel" . "arstoffel@gmail.com")) :maintainer '("Augusto Stoffel" . "arstoffel@gmail.com") :keywords '("languages" "processes") :url "https://github.com/astoff/drepl")
